require "lib/util.l"
require "lib/string.l"
require "lib/parser.l"

selected=1
chosen=false
state=0
quit=false
Wlan.init()
configs = Wlan.getConnectionConfigs()
timeout=80

function chooseNetwork()
  Wlan.term()
  Wlan.init()
  configs = Wlan.getConnectionConfigs()

  local prompt=false
  while true do
    name,nstate,button = choose("Select a Network Connection",configs,nil,nil,1)
    if quit then break end
    if button==1 then
      prompt= confirm("Use connection " .. name .. "?")==false
      local yes=0
      if prompt==true then
        yes=1
      else
        break
      end
    end
    showMsg('Rechoose connection',400)
  end
  if not prompt and nstate ~= -1 then
    nstate=nstate+1
    showMsg("Connecting from " .. name,1)
    return Wlan.useConnectionConfig(tonumber(nstate))
  end
end

function getName(url)
  local str = string.gsub (url, "+", " ") 
  str = string.gsub (str, "%%(%x%x)", function(h) return string.char(tonumber(h,16)) end) 
  str = string.gsub (str, "\r\n", "\n")
  url=str
  parts=string.split(url,'/')
  return parts[table.getn(parts)]
end

function getText(host,url)
  showMsg('Downloading bookmark')
  local tries=0
  local socket, error = Socket.connect(host,80)
  local abort = false
  local headerDone = false
  local contents=""
  local buffer=''
  while not socket:isConnected() do
    tries = tries+1
    System.sleep(100) 
    if tries > timeout then
      showMsg("Timed out - please retry\n")
      System.sleep(1000)
      quit=true
      return nil --no connect
    end
  end
  bytesSent = socket:send("GET "..url.." HTTP/1.0\r\n")
  bytesSent = socket:send("host: "..host.."\r\n\r\n")
  local started=false
  local size=0
  local total=0
  local header=""
  while true do
    buffer = socket:recv()
    if string.len(buffer) > 0 then
      total=total+string.len(buffer)
      started=true
      if headerDone then
        contents = contents .. buffer
      else
        header = header .. buffer
      end
      if string.contains(buffer,'\r\n\r\n') then
        parts=string.split(buffer,"\r\n\r\n")
        headerDone=true
        large = parts[2]
        headerLines = string.split(header,"\n")
        for i=1,table.getn(headerLines) do
          l= string.trim(headerLines[i])
            leng = string.contains(l,"Content(%-)Length(.+)")
            if leng then
              local sp = string.split(l," ")
              size = tonumber(sp[2])
              if size>1024*200 then
                local cont=confirm('File is '..(size/1024/1024)..' Mb.  Continue?')
                if cont==false then
                  clear()
                  return nil
                end
              end
            end
        end
      end
    elseif started and headerDone and total>=size then
        break
    end
    if Controls.read():start() then
      abort=true
      break 
    end
  end
  clear()
  if abort then
    return nil --save aborted
  else
    return contents --success
  end
end

--pass the host name and port to connect to (optional, defaults to 80)
function connect(host,port)
  if port==nil then
   port=80
  end
  local tries=0
  local socket, error = Socket.connect(host,port)
  local conn={}
  conn.host=host
  System.powerTick()
  showMsg('Connecting to ' .. host)
  while not socket:isConnected() do
    tries = tries+1
    if tries > timeout then
      showMsg("Timed out - please retry\n")
      System.sleep(100)
      return nil --no connect
    end
    System.sleep(100) 
  end
  conn.sock=socket
  conn.open=true
  return conn
end

--pass in an open connection and a path to the file
function getHeader(conn,path)
    showMsg('Retrieving header info')
    bytesSent = conn.sock:send("GET "..path.." HTTP/1.0\r\n")
    bytesSent = conn.sock:send("host: "..conn.host.."\r\n\r\n")

    local size=0
    local total=0
    --local pad=Pad.new()
    local buffer=''
    local current=0
    local h_lines=""
    local header={}
    
    --+download loop
    clear()
    while true do
      Pad:Update()
      buffer = conn.sock:recv()
      if string.len(buffer)>0 then
        if string.contains(buffer,'\r\n\r\n') then
          local parts=string.split(buffer,'\r\n\r\n')
          h_lines= h_lines .. parts[1]
          header.content=parts[2]
          break
        else
          h_lines= h_lines .. buffer
        end
      end
      if Pad.tSt then
        break
      end
      System.sleep(100)
    end
    header.size=string.len(h_lines)+4
    local parts=string.split(h_lines,'\n')
    for i=1,table.getn(parts) do
      --gprintln('header line ' .. parts[i])
      if string.contains(parts[i],':') then
        local tmp=string.split(parts[i],':')
        local val = table.concat(tmp,':',2)
        header[string.lower(tmp[1])]=string.lower(val)
      end
    end
    return header
end

function showpage(conn,path)
  local header = getHeader(conn,path)
  local buffer = ''
  local s = { header.content }
  local recvd = string.len(header.content)
  local size = tonumber(header['content\-length']) or 9999

  while recvd < size do
    buffer = conn.sock:recv()
    recvd = recvd + string.len(buffer)
    if string.len(buffer)>0 then
      table.insert(s,buffer)
    else
      break
    end
  end
  buffer = table.concat(s)
  return buffer
end

function getFile(conn,r_path,l_path)
  local fw = 12
  local shortName=getName(l_path)
  local tries=0
  local socket = conn.sock
  local abort = false
  local headerDone = false
  local timer=Timer.new()
  local remove=false
  while not socket:isConnected() do
    tries = tries+1
    if tries > timeout then
      showMsg("Timed out - please retry\n")
      System.sleep(500)
      return 1 --no connect
    end
    System.sleep(100) 
  end
  System.powerTick()
  local outf=io.open(l_path,"w")
  if outf then
    bytesSent = socket:send("GET "..r_path.." HTTP/1.0\r\n")
    bytesSent = socket:send("host: "..conn.host.."\r\n\r\n")

    local snum=100
    local started=false
    local size=99999999
    local total=1
    local last=0
    local current=0
    local header=""
    local kbps=0
    local min = 1
    local sec = 1
    local per = 0
    local secleft = 1
    local s = {}

    --download loop
    timer:start()
    while true do

      Pad:Update()

      buffer = socket:recv()

      --get info from header
      if string.len(buffer) > 0 then
        started=true
        if headerDone then
          table.insert(s,buffer)
        for i=1,snum do
            table.insert(s,socket:recv())
        end
          buffer = table.concat(s)
          total=total+string.len(buffer)
          outf:write(buffer)
          s = {}
        else
          header = header .. buffer
        end
        if string.contains(buffer,'\r\n\r\n') then
          parts=string.split(buffer,"\r\n\r\n")
          headerDone=true
          buffer = parts[2]
--          outf:write(buffer)
          headerLines = string.split(header,"\n")
          for i=1,table.getn(headerLines) do
            l= string.trim(headerLines[i])
              leng = string.contains(l,"Content(%-)Length(.+)")
              if leng then
                local sp = string.split(l," ")
                size = tonumber(sp[2])
                if size>1024*1024*10 then
                  clear()
                  local cont=confirm('File is '..(size/1024/1024)..'  Continue?')
                  if cont==false then
                    clear()
                    abort=true
                    remove=true
                    break
                  else
                    outf:write(buffer)
                  end
                else
                  outf:write(buffer)
                end
              end
          end
        end

        if abort then
          break
        end

        if string.len(buffer)>0 and headerDone then
         System.powerTick()
           screen:clear()
           kbps= (total/timer:start())
           secleft=math.floor((size-total)/(kbps*1024))
           min=string.format("%d", secleft/60 )
           sec=string.format("%d", ((secleft/60)-min)*60 )
           per=string.format("%d",((total/size)*100))
           clear()
           os:print(fw*2,fw*2, string.format("%.2f",tonumber(size-total)/1024/1024) .." megabytes left of ".. string.format("%.2f",size/1024/1024),yellow)
           os:print(fw*2,fw*3, per .. "% of file: " .. shortName,yellow)
           os:print(fw*2,fw*4, string.format("%.1f",kbps) .. " kbps  || socket buffer " .. snum,yellow)
           os:print(fw*2,fw*5, System.powerGetBatteryLifePercent() .. '% battery remaining',yellow)
           os:print(fw*2,fw*6, min .. " minute(s) " .. sec .. " seconds until complete",yellow)
           os:fillRect(fw*2,(fw*6)+10,100*1.5,fw,yellow)
           os:fillRect(fw*2,(fw*6)+10,math.floor(per)*1.5,fw,red)
           os:print(fw*2,fw*9, "Start : Abort download",yellow)
           os:print(fw*2,fw*10, " L/R : Adjust socket buffer",yellow)
           screen:blit(0,0,os)
           screen.waitVblankStart()
           screen.flip()
        end
      elseif started then
        subd=string.format("%.2f",(((total+string.len(buffer))/size)*100))
        clear()
        if tonumber(subd)>=98 then
          break
        end
      end
      if Pad.tSt then
        abort=true
        clear()
        remove=confirm("Would you like to delete the partial file?")
        if remove then
          outf:close()
          System.removeFile(l_path)
        end
        break
      elseif Pad.tSl then
        screen:save('download.png')
      elseif Pad.Ll then
        if snum>3 then
          snum=snum-1
        end
      elseif Pad.Lr then
        snum=snum+1
      end
    end
    if string.len(buffer) > 0 and not remove then
      outf:write(buffer)
      showMsg("Done!! ")
      if go == nil then
        System.sleep(2000)
      end
      outf:flush()
      outf:close()
    end
    clear()
    if abort then
      return 2 --save aborted
    else
      return 0 --success
    end
  else
    gprint("out file error\n")
    return 3 --error opening save file
  end
end


--[[
function getFile_broke(conn,r_path,l_path)
  fw=12
  ms:setPixelSizes(0, fw)
  local shortName=getName(l_path)
  local abort = false
  local timer=Timer.new()
  local remove=false
  local outf=io.open(l_path,"w")
  if outf then

    local large = ""
    local bnum=13
    local snum=256
    local bufsize=1024*bnum
    local started=false
    local size=0
    local total=0
    local last=0
    local current=0
    local header=""
    local kbps=0
    local pad=Pad.new()
    local header=getHeader(conn,r_path)
    local buffer=header.content
    local s={ buffer }
    

    --+download loop
    while true do
      started=true
      timer:start()
      pad:Update()

       --+buffer the socket data
        buffer = conn.sock:recv()

        if string.len(buffer)>0 then
          table.insert(s,buffer)
          for k=1,snum do
              buffer = conn.sock:recv()
              table.insert(s,buffer)
          end
       end

       --+push the tmp data back to the buffer
        buffer=table.concat(s)

        --+add the buffer size to total
        total=total+string.len(buffer)
        
        if abort then
          break
        end

        System.powerTick()
        screen:clear()
        kbps= total/timer:start()
        secleft=math.floor((size-total)/(kbps*1024))
        min=string.format("%d", secleft/60 )
        sec=string.format("%d", ((secleft/60)-min)*60 )
        per=string.format("%.2f",((total/size)*100))
        screen:fontPrint(ms,fw*2,fw*2, string.format("%.2f",tonumber(size-total)/1024/1024) .." megabytes left of ".. string.format("%.2f",size/1024/1024),yellow)
        screen:fontPrint(ms,fw*2,fw*3, per .. "% of file: " .. savename,yellow)
        screen:fontPrint(ms,fw*2,fw*4, kbps .. " kbps    buffer size: ".. bufsize .. '   socketbuffer: '..snum,yellow)
        screen:fontPrint(ms,fw*2,fw*5, System.powerGetBatteryLifePercent() .. '% battery remaining',yellow)
        screen:fontPrint(ms,fw*2,fw*6, min .. " minute(s) " .. sec .. " seconds until complete",yellow)
        screen:fillRect(fw*2,(fw*6)+10,100*1.5,fw,yellow)
        screen:fillRect(fw*2,(fw*6)+10,math.floor(per)*1.5,fw,red)
        screen:fontPrint(ms,fw*2,fw*9, "Start : Abort download",yellow)
        screen:fontPrint(ms,fw*2,fw*10, " L/R  : Adjust socket buffer",yellow)
        screen:fontPrint(ms,fw*2,fw*11, " U/D  : Adjust file buffer",yellow)
        screen.flip()
        outf:write(buffer)

      --elseif started then
      if string.len(buffer)==0 then
        subd=string.format("%.2f",(((total)/size)*100))
        clear()
        if tonumber(subd)>=98 then
          break
        end
      end  --end buffering loop

      if pad.tSt then
        abort=true
        clear()
        remove=confirm("Would you like to delete the partial file?")
        if remove then
          outf:close()
          System.removeFile(l_path)
        end
        break
      elseif pad.Ll then
        if snum>3 then
          snum=snum-1
        end
      elseif pad.Lr then
        snum=snum+1
      elseif pad.Lu then
        bnum=bnum+1
        bufsize=1024*bnum
      elseif pad.Ld then
        if bnum>1 then
          bnum=bnum-1
          bufsize=1024*bnum
        end
      end
    end  --end download loop

    if not remove then
      fw=8
      outf:write(buffer)
      showMsg("Done!! ")
      if go == nil then
        System.sleep(2000)
      end
      outf:flush()
      outf:close()
    end

    clear()
    if abort then
      return 2 --save aborted
    else
      return 0 --success
    end
  else
    gprint("out file error\n")
    return 3 --error opening save file
  end
end
]]

function spliturl(url)
  if not string.contains(url,'http://') then
    url = 'http://' .. url
  end
  local st=string.find(url,'//')
  local host_=string.sub(url,st+2)
  local host=''
  local filename=''
  if string.contains(host_,'/') then
    host=string.sub(host_,1,string.find(host_,'/')-1)
    filename=string.sub(host_,string.len(host)+1,-1)
  else
    host=host_
    filename='/'
  end
  return host,filename
end